import React, { useState, useCallback, useEffect } from 'react';
import ReactDOM from 'react-dom';
import { csv, scaleLinear, scaleOrdinal, max, format, extent } from 'd3';
import ReactDropdown from 'react-dropdown';
import { useData } from './useData';
import { AxisBottom } from './AxisBottom';
import { AxisLeft } from './AxisLeft';
import { Marks } from './Marks';
import { ColorLegend } from './ColorLegend';

const extraMenuBuffer = 30;

const width = 960;
const menuHeight = 80;
const height = 500 - menuHeight - extraMenuBuffer;
const margin = { top: 20, right: 200, bottom: 65, left: 90 };
const xAxisLabelOffset = 50;
const yAxisLabelOffset = 45;
const fadeOpacity = 0.2;

const attributes = [
  { value: 'Year', label: 'Year' },
  { value: 'RaceorEthnicity', label: 'Race or Ethnicity' },
  { value: 'Sex', label: 'Sex' },
  { value: 'AgeGroup', label: 'Age Group' },
  { value: 'FirearmDeaths', label: 'Firearm Deaths' },
  { value: 'SuicideDeaths', label: 'Suicide Deaths' },
  { value: 'AlcoholRelatedDeaths', label: 'Alcohol-Related Deaths' },
  
  { value: 'male', label: 'Male' },
  { value: 'female', label: 'Female' },
  { value: 'all', label: 'All' },
  
  { value: 'WhiteNonHispanic', label: 'White Non Hispanic' },
  { value: 'OtherNonHispanic', label: 'Other Non Hispanic' },
  { value: 'NotStated', label: 'Not Stated' },
  { value: 'Hispanic', label: 'Hispanic' },
  { value: 'BlackNonHispanic', label: 'Black Non Hispanic' },
  
  { value: '<1', label: '<1' },
  { value: '1-9', label: '1-9' },
  { value: '10-19', label: '10-19' },
  { value: '20-24', label: '20-24' },
  { value: '25-34', label: '25-34' },
  { value: '35-44', label: '35-44' },
  { value: '45-54', label: '45-54' },
  { value: '55-64', label: '55-64' },
  { value: '65-74', label: '65-74' },
  { value: '75-84', label: '75-84' },
  { value: '85+', label: '85+' }
];

const sexAttributes = [
  { value: 'male', label: 'Male' },
  { value: 'female', label: 'Female' },
  { value: 'all', label: 'All' }
];

const ethnicityAttributes = [
  { value: 'WhiteNonHispanic', label: 'White Non Hispanic' },
  { value: 'OtherNonHispanic', label: 'Other Non Hispanic' },
  { value: 'NotStated', label: 'Not Stated' },
  { value: 'Hispanic', label: 'Hispanic' },
  { value: 'BlackNonHispanic', label: 'Black Non Hispanic' },
  { value: 'all', label: 'All' }
];

const ageAttributes = [
  { value: '<1', label: '<1' },
  { value: '1-9', label: '1-9' },
  { value: '10-19', label: '10-19' },
  { value: '20-24', label: '20-24' },
  { value: '25-34', label: '25-34' },
  { value: '35-44', label: '35-44' },
  { value: '45-54', label: '45-54' },
  { value: '55-64', label: '55-64' },
  { value: '65-74', label: '65-74' },
  { value: '75-84', label: '75-84' },
  { value: '85+', label: '85+' },
  { value: 'all', label: 'All' }
];

const deathAttributes = [
  { value: 'FirearmDeaths', label: 'Firearm Deaths' },
  { value: 'SuicideDeaths', label: 'Suicide Deaths' },
  { value: 'AlcoholRelatedDeaths', label: 'Alcohol-Related Deaths' }
];





const getLabel = value => {
  for (let i = 0; i < attributes.length; i++) {
    if (attributes[i].value === value) {
      return attributes[i].label;
    }
  }
};


const App = () => {
  const data = useData();
  const [hoveredValue, setHoveredValue] = useState(null);


  // X axis (constant years)
  const initialXAttribute = 'Year';
  const [xAttribute, setXAttribute] = useState(initialXAttribute);
  const xValue = d => d[xAttribute];
  const xAxisLabel = getLabel(xAttribute);
  
  
  // The Menus
  const initialSexAttribute = 'All';
  const [sexAttribute, setSexAttribute] = useState(initialSexAttribute);
  const sexValue = d => d[sexAttribute];
  const sexAxisLabel = getLabel(sexAttribute);
  
  const initialEthnicityAttribute = 'All';
  const [ethnicityAttribute, setEthnicityAttribute] = useState(initialEthnicityAttribute);
  const ethnicityValue = d => d[ethnicityAttribute];
  const ethnicityAxisLabel = getLabel(ethnicityAttribute);
  
  const initialAgeAttribute = 'All';
  const [ageAttribute, setAgeAttribute] = useState(initialAgeAttribute);
  const ageValue = d => d[ageAttribute];
  const ageAxisLabel = getLabel(ageAttribute);

  // Y axis menu
  const initialYAttribute = 'AlcoholRelatedDeaths';
  const [yAttribute, setYAttribute] = useState(initialYAttribute);
  const yValue = d => d[yAttribute];
  const yAxisLabel = getLabel(yAttribute);
  
  
  // Load the data after all the hooks are rendered.
  if (!data) {
    return <pre>Loading...</pre>;
  }
  
  
  const selected_age_group = 'Total';
  
  const sexColorValue = d => d.Sex;
  const sexColorLegendLabel = 'Sex';
  
  const filteredData = data.filter(d => hoveredValue === sexColorValue(d));
  
  const circleRadius = 7;



  const innerHeight = height - margin.top - margin.bottom;
  const innerWidth = width - margin.left - margin.right;
  const siFormat = format('t');
  const xAxisTickFormat = tickValue => siFormat(tickValue).replace('G', 'B');

  
  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();

  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0]);
  
  const sexColorScale = scaleOrdinal()
    .domain(data.map(sexColorValue))
    .range(['#FF0000', '#0000FF']);
  

  
  return (
    <>
      <div className="menus-container">
        <span className="dropdown-label">Sex</span>
        <ReactDropdown
          options={sexAttributes}
          value={sexAttribute}
          onChange={({ value }) => setSexAttribute(value)}
        />
        <span className="dropdown-label">Ethnicity</span>
        <ReactDropdown
          options={ethnicityAttributes}
          value={ethnicityAttribute}
          onChange={({ value }) => setEthnicityAttribute(value)}
        />
        <span className="dropdown-label">Age Group</span>
        <ReactDropdown
          options={ageAttributes}
          value={ageAttribute}
          onChange={({ value }) => setAgeAttribute(value)}
        />
        <span className="dropdown-label">Deaths</span>
        <ReactDropdown
          options={deathAttributes}
          value={yAttribute}
          onChange={({ value }) => setYAttribute(value)}
        />
      </div>
      
      <svg width={width} height={height}>
        <g transform={`translate(${margin.left},${margin.top})`}>
          <AxisBottom
            xScale={xScale}
            innerHeight={innerHeight}
            tickFormat={xAxisTickFormat}
            tickOffset={5}
          />
          <text
            className="axis-label"
            textAnchor="middle"
            transform={`translate(${-yAxisLabelOffset},${innerHeight /
              2}) rotate(-90)`}
          >
            {yAxisLabel}
          </text>
          <AxisLeft yScale={yScale} innerWidth={innerWidth} tickOffset={5} />
          <text
            className="axis-label"
            x={innerWidth / 2}
            y={innerHeight + xAxisLabelOffset}
            textAnchor="middle"
          >
            {'Year'}
          </text>
          
          <g transform={`translate(${innerWidth + 60}, 60)`}>
            <text
              x={35}
              y={-25}
              className="axis-label"
              textAnchor="middle"
            >
              {sexColorLegendLabel}
            </text>
            <ColorLegend
              tickSpacing={22}
              tickTextOffset={12}
              tickSize={circleRadius}
              colorScale={sexColorScale}
              onHover={setHoveredValue}
              hoveredValue={hoveredValue}
              fadeOpacity={fadeOpacity}
            />
          </g>
          
          <g opacity={hoveredValue ? fadeOpacity : 1}>
            <Marks
              data={data}
              xScale={xScale}
              yScale={yScale}
              selectedSex={sexValue}
              sexColorScale={sexColorScale}
              sexColorValue={sexColorValue}
              yValue={yValue}
              selected_age_group={selected_age_group}
              tooltipFormat={xAxisTickFormat}
              circleRadius={circleRadius}
            />
        	</g>	
          
          <Marks
            data={filteredData}
            xScale={xScale}
            yScale={yScale}
            selectedSex={sexValue}
            sexColorScale={sexColorScale}
            sexColorValue={sexColorValue}
            yValue={yValue}
            selected_age_group={selected_age_group}
            tooltipFormat={xAxisTickFormat}
            circleRadius={circleRadius}
          />
          
        </g>
        
      </svg>
    </>
  );
};
const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
